# Keras and Deep Deterministic Policy Gradient to control an OMNeT++ network simulator

## How to train?

### Single run with fixed parameters
Reads configuration from ```DDPG.json```

```
python3 ddpg.py
```

### Experiment with variation in parameters
Reads basic configuration from ```EXPERIMENT.json``` and changes some parameters according to what is defined in function experiment(), ```cluster``` is needed to choose between ```local``` and ```arvei``` execution

```
python3 experiment.py cluster EXPERIMENT.json
```

## How to play?

### Single run with fixed parameters
Reads configuration (ddpg, neural network weights, etc.) from ```folder```

```
python3 ddpg.py play folder
```

---

### EXAMPLE JSON CONFIG

```
{
    "ACTIVE_NODES": 3,                                          # number of active nodes in the network
    "ACTUM": "NEW",                                             # action: NEW or DELTA
    "BATCH_SIZE": 50,                                           # size of learning batch
    "BN": "reward",                                             # batch normalization
    "BUFFER_SIZE": 2000,                                        # max size of replay buffer
    "ENV": "label",                                             # "label" or "balancing"
    "EPISODE_COUNT": 10,                                        # number of episodes
    "EXPLORE": 0.8,                                             # exploration: rate if <=1, number of steps otherwise
    "GAMMA": 0.99,                                              # discount factor
    "HACTI": "selu",                                            # non-linear activation function for hidden layers
    "HIDDEN1_UNITS": 300,                                       # neurons of layer 1
    "HIDDEN2_UNITS": 600,                                       # neurons of layer 2
    "LRA": 0.0001,                                              # learning rate of the actor network
    "LRC": 0.001,                                               # learning rate of the critic network
    "MAX_DELTA": 0.1,                                           # max allowed change if actum is DELTA
    "MAX_STEPS": 1000,                                          # number of steps per episode
    "MU": 0.0,                                                  # Ornstein-Uhlenbeck process' μ
    "PRAEMIUM": "MAX",                                          # reward function
    "PRINT": true,                                              # verbosity
    "ROUTING": "Linkweight",                                    # "Balancer", "Linkweight", "Pathchoice"
    "RSEED": null,                                              # random seed: null or number
    "SIGMA": 0.07,                                              # Ornstein-Uhlenbeck process' σ
    "STATUM": "T",                                              # state representation: T or RT
    "TAU": 0.001,                                               # soft target update
    "THETA": 0.03,                                              # Ornstein-Uhlenbeck process' θ
    "TRAFFIC": "STAT:-1,4.9,4.9,4.9,-1,4.9,4.9,4.9,-1",         # traffic: static or changing (~randomly)
    "U_ROUTING": "3,0,2,0,2,3,1,0,3"                            # routing of underlay network
}
```

---

author: giorgio@ac.upc.edu

* [Keras](https://keras.io/)
* [DDPG](https://arxiv.org/abs/1509.02971)
* [OMNeT++](https://omnetpp.org/)
